/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   02.03.2015
 * @brief  Base interface for SIB access point.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * Access point implements access method to the SIB (TCP, UDP, Bluetooth)
 * and processes requests from agents and sends responses to agents.
 * The SIB uses access points as plug-ins.
 * This file describe interface for access point that is used by SIB.
 *
 * Web-site: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2015-2016 by PetrSU.
 */


#ifndef CUTESIB_ACCESSPOINTINTERFACE_H
#define CUTESIB_ACCESSPOINTINTERFACE_H

#include "accesspointinterface_global.h"

#include <QtCore/QSharedPointer>

#include "client.h"
#include "request.h"

// Define indetifier for the access point interface.
#define AccessPointInterface_iid "org.fruct.oss.Projects.CuteSIB.AccessPointInterface"

// Protocols id/names.
#define CUTESIB_PROTOCOL_SSAPXML "SSAP/XML"
#define CUTESIB_PROTOCOL_HTTPSPARQL "HTTP/SPARQL"
#define CUTESIB_PROTOCOL_DISCOVERY "CuteSIB/Discovery"

namespace cutesib {
namespace access_point {

//TODO: how to change: dosent store interafce in the request
// Forward declaration to compile (eliminate cross .h includes)
class Request;
class Client;

/**
 * @brief The AccessPointInterface class is used as an interface
 * to represent access poitn for SIB.
 */
class ACCESSPOINTINTERFACESHARED_EXPORT AccessPointInterface
{
public:
    /**
     * @brief Empty virtual destructor for an inheritance.
     */
    virtual ~AccessPointInterface() {}

    /**
     * @brief Sets parameters of the access point.
     * Format of parameter is not defined.
     * Base form is 'key:value;key:value'.
     * @param accessPointId is an identifier for access point.
     * @param parameters of the access point.
     * @see id()
     */
    virtual void initialize(QString accessPointId, QString parameters) = 0;

    /**
     * @brief Gets identifier of access point instance.
     * SIB sets this id and uses it to control access point.
     * This id does not used by access point.
     * @return SIB config identifier.
     * @see setParameters
     */
    virtual QString id() = 0;

    /**
     * @brief Starts the access point.
     * @return 0 on success or -1 otherwise.
     */
    virtual int start() = 0;

    /**
     * @brief Stops work of access point.
     * The access point stops listening clients, close connection with current clients,
     * remove temorary information about clients connection etc.
     * After calling this method the access point can be started again (#start).
     */
    virtual void stop() = 0;

    /**
     * @brief Shutdowns the access point.
     * After calling this method, access point will no longer be used
     * and can be deleted in any time.
     */
    virtual void shutdown() = 0;

    /**
     * @brief Starts to check connection state of a client (KeepAlive)
     * The client will be checking with some frequency.
     * When the client is no longer available, then the signal
     * (#clientDisconnected()) will be emitted.
     * The method does nothing if client connection state is already checking.
     * @param client to start checking connection.
     * @see clientDisconnected()
     */
    virtual void startChecking(QSharedPointer<Client> client) = 0;

    /**
     * @brief Stops to check connection state of a client.
     * The method does nothing if client connection state is not checking.
     * @param client to stop checking connection.
     */
    virtual void stopChecking(QSharedPointer<Client> client) = 0;

protected:
    /**
     * @brief Protected empty constructor to prevent instantiating.
     */
    AccessPointInterface() {}


signals:
    /**
     * @brief Signals when a client disconnected from the access point.
     * When a client is disconnected (itself or network error),
     * then a client will not be used by the access point.
     * The client instance must be deleted in the slot.
     * @param client that was disconnected.
     */
    virtual void clientDisconnected(QSharedPointer<Client> client) = 0;

    /**
     * @brief Signals when a new client connected to the access point.
     * @param client that have been connected to access point.
     */
    virtual void clientConnected(QSharedPointer<Client> client) = 0;

    /**
     * @brief Signals when a client sends a request to access point.
     * @param client that sends a request.
     * @param data that were sent by the client.
     */
    virtual void requestReceived(QSharedPointer<Request> request) = 0;

    /**
     * @brief Signals when the access poin stops working.
     * SIB uses this signal when access poin working in own thread (async AP).
     */
    virtual void stopped() = 0;
};

} // Namespace access_point.
} // Namespace cutesib.

// Declare QT interface.
Q_DECLARE_INTERFACE(cutesib::access_point::AccessPointInterface, AccessPointInterface_iid)

#endif // CUTESIB_ACCESSPOINTINTERFACE_H
