/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   08.03.2015
 * @brief  Class to manage access points and protocol handlers.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * Manger to loads access points (built-in and static) in runtime
 * and to link access points with rotocol handlers.
 *
 * Contact: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2015-2016 by PetrSU.
 */


#include "accesspointmanager.h"

#include <iostream>

#include <QtCore/QString>
#include <QtCore/QDir>
#include <QtCore/QPluginLoader>
#include <QtCore/QDebug>

#include "accesspointloaderinterface.h"
#include "ssaphandler.h"
#include "logger.h"


/*****************************************************************************/
/************************** Constructors/destructors *************************/

cutesib::AccessPointManager::AccessPointManager(SibConfig *config, QObject *parent) : QObject(parent)
{
    config_ = config;
}

cutesib::AccessPointManager::~AccessPointManager()
{

}


/*****************************************************************************/
/****************************** Public methods *******************************/

void cutesib::AccessPointManager::load()
{
    foreach(QString id, config_->getAccessPointsConfigIds()) {

        // Try to load static (built-in) access point.
        QString staticId = config_->getAccessPointStaticId(id);

        if (!staticId.isEmpty()) {
            loadStatic(id, staticId);
            continue;
        }

        // Try to load non-static (dynamic) access point.
        QString fullpath = config_->getAccessPointPath(id);

        QPluginLoader *pluginLoader = new QPluginLoader(fullpath, this);
        pluginLoaders_.append(pluginLoader);

        QObject *plugin = pluginLoader->instance();

        if (pluginLoader->isLoaded()) {
            auto *accessPointLoader = qobject_cast<AccessPointLoaderInterface *>(plugin);
            pointLoaders_.insert(id, accessPointLoader);
        } else {
            qWarning() << "Loading access point error:" << pluginLoader->errorString();
        }


    }
}


void cutesib::AccessPointManager::unloadAll()
{
    foreach (QPluginLoader *loader, pluginLoaders_) {
        loader->unload();
        delete loader;
    }

    pluginLoaders_.clear();
    pointLoaders_.clear();
}


void cutesib::AccessPointManager::setAccessPoints(cutesib::ProtocolHandler *handler)
{
    foreach (QString id, pointLoaders_.keys()) {
        AccessPointLoaderInterface *loader = pointLoaders_.take(id);

        foreach(QString protocol, handler->protocols()) {
            if (loader->protocols().contains(protocol, Qt::CaseInsensitive)) {
                auto accessPoint = loader->create(id, config_->getAccessPointParameters(id));
                handler->connectAccessPoint(accessPoint, config_->getAccessPointAsyncFlag(id));
                break;
            }
        }
    }
}


/*****************************************************************************/
/***************************** Private methods *******************************/

void cutesib::AccessPointManager::loadStatic(QString id, QString staticId )
{
    foreach (QObject* plugin, QPluginLoader::staticInstances()) {
        auto *loader = qobject_cast<cutesib::access_point::AccessPointLoaderInterface *>(plugin);
        if (loader->id() == staticId) {
            pointLoaders_.insert(id, loader);
        }
    }
}
