/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   08.03.2015
 * @brief  Class to manage access points and protocol handlers.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * Manger to loads access points (built-in and static) in runtime
 * and to link access points with rotocol handlers.
 *
 * Contact: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2015-2016 by PetrSU.
 */


#ifndef CUTESIB_ACCESSPOINTMANAGER_H
#define CUTESIB_ACCESSPOINTMANAGER_H

#include "accesspointloaderinterface.h"

#include <QObject>
#include <QString>
#include <QMap>
#include <QPluginLoader>

#include "ssaphandler.h"
#include "sibconfig.h"


namespace cutesib {

/**
 * @brief The AccessPointManager class manages
 * access points (built-in and static) in runtime
 * and links access points with rotocol handlers.
 */
class AccessPointManager : public QObject
{
    Q_OBJECT

public:
    /**
     * @brief AccessPointManager class constructor.
     * @param config
     * @param parent
     */
    AccessPointManager(SibConfig *config, QObject *parent = 0);

    /**
     * @brief AccessPointManager destructor.
     * Unload all plugins (access points).
     */
    ~AccessPointManager();

    /**
     * @brief Loads access point plugins based on config information.
     * Gets informtion from config and tries to load static (built-in)
     * or dynamic (dynamic library) plugins.
     * @param config of the SIB.
     */
    void load();

    /**
     * @brief Unload all plugins (access points).
     */
    void unloadAll();

    /**
     * @brief Sets access points for given handler.
     * Sets access points for handlers with corresponding protocols.
     * @param handler to set access points.
     */
    void setAccessPoints(ProtocolHandler *handler);

private: // Methods.
    /**
     * @brief Loads static plugin (access point).
     * @param id from config.
     * @param staticId of access point to configure.
     */
    void loadStatic(QString id, QString staticId);

private: // Fields.
    SibConfig *config_;         ///< SIB config.
    QList<QPluginLoader*> pluginLoaders_;   ///< Stores plugin loades.
    QMap<QString, AccessPointLoaderInterface*> pointLoaders_;   ///< Stores access point loades  by config ids (in config file '[AccessPoint:<id>]').

};

} // Namespace cutesib.

#endif // CUTESIB_ACCESSPOINTMANAGER_H
