/**
* @author Ivan V. Galov <galov@cs.karelia.ru>
* @date   18.04.2016
* @brief  Handler for processing basic smart space operations.
*
* This file is part of the CuteSIB project.
*
* @section LICENSE
*
* CuteSIB is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* CuteSIB is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with CuteSIB; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor,
* Boston, MA  02110-1301  USA
*
* @section DESCRIPTION
*
* Handles basic operations of SSAP protocol: query, insert, remove, update.
*
* Web-site: http://oss.fruct.org/wiki/SmartSpace-platform
* Copyright (C) 2015-2016 by PetrSU.
**/

#include "basicoperationhandler.h"

#include "tests/stopwatch.h"
#include "context.h"

cutesib::BasicOperationHandler::BasicOperationHandler(QObject *parent) : OperationHandler(QString(BASICOPERATIONHANDLER_ID), parent)
{

}

cutesib::SibError cutesib::BasicOperationHandler::initialize(cutesib::Context *context)
{
    SibModule::initialize(context);
    triplestore_ = this->context()->triplestore();
    RDF_world = triplestore_->getRDFWorld();
    RDF_model = triplestore_->getRDFModel();

    return SibError::NONE;
}

cutesib::CommandResponse *cutesib::BasicOperationHandler::insert(CommandRequest *command)
{
    rdfInsert(command->insertTriples());

    CommandRequest *insertCommand = new CommandRequest(CommandType::INSERT, this->id());
    insertCommand->setInsertTriplesS(command->insertTriplesS());
    emit storeUpdated(insertCommand);
    delete insertCommand;

    Stopwatch::stopAndWrite("insert");

    if (command->needConfirmation())
        return new CommandResponse(command, true);
    else
        return NULL;
}

cutesib::CommandResponse* cutesib::BasicOperationHandler::query(CommandRequest* command)
{
    QList<cutesib::Triple *> queryTriples = command->queryTriples();

    //Rdf Redland Initialize..
    librdf_statement* 	statement ;
    librdf_statement*	partial_statement;
    librdf_stream* stream;

    librdf_node* sub_node ;
    librdf_node* pred_node;
    librdf_node* obj_node ;

    QList<cutesib::Triple *> tripleList;
    Triple* resultTriple;

    foreach (Triple* triple, queryTriples) {
        // Redland query with wildcard support
        partial_statement=librdf_new_statement(RDF_world);

        if ( (strcmp((char*) triple->subject().toUtf8().data(), wildcard1) ==0) || ( (strcmp((char*) triple->subject().toUtf8().data() ,wildcard2) ==0) ))
        {	librdf_statement_set_subject(partial_statement, NULL);
        }
        else
        {	librdf_statement_set_subject(partial_statement, librdf_new_node_from_uri_string(RDF_world,  (const unsigned char*) triple->subject().toUtf8().data()));
        }

        if ( (strcmp((char*) triple->predicate().toUtf8().data(), wildcard1) ==0) || ( (strcmp((char*) triple->predicate().toUtf8().data() ,wildcard2) ==0) ))
        {	librdf_statement_set_predicate(partial_statement, NULL);
        }
        else
        {	librdf_statement_set_predicate(partial_statement, librdf_new_node_from_uri_string(RDF_world, (const unsigned char*) triple->predicate().toUtf8().data()));
        }

        if ( (strcmp((char*) triple->object().toUtf8().data(), wildcard1) ==0) || ( (strcmp((char*) triple->object().toUtf8().data() ,wildcard2) ==0) ))
        {	librdf_statement_set_object(partial_statement, NULL);
        }
        else
        {
            if (triple->objectType() == RdfTripleType::URI)
            {	librdf_statement_set_object(partial_statement,
                                            librdf_new_node_from_uri_string(RDF_world, (const unsigned char*) triple->object().toUtf8().data()));
            }
            else
            {	librdf_statement_set_object(partial_statement,
                                            librdf_new_node_from_literal(RDF_world, (const unsigned char*) triple->object().toUtf8().data(), NULL, 0));
            }
        }

        /*if (p->virtuoso)
        stream=librdf_model_find_statements_in_context(p->RDF_model, partial_statement, p->virtuoso_context);
    else*/
        stream=librdf_model_find_statements(RDF_model, partial_statement);

        librdf_free_statement(partial_statement);


        while(!librdf_stream_end(stream)) {
            statement=librdf_stream_get_object(stream);


            ///////////////////////////////////////////////////////////////////////////////////////////

            sub_node  = librdf_statement_get_subject(statement);
            pred_node = librdf_statement_get_predicate(statement);
            obj_node  = librdf_statement_get_object(statement);

            if (( librdf_node_is_blank(sub_node))||( librdf_node_is_blank(obj_node)))
            {
                librdf_stream_next(stream);
                continue;
            }


            char* sub = (char*) librdf_uri_as_string( librdf_node_get_uri(sub_node));
            char* pred = (char*) librdf_uri_as_string( librdf_node_get_uri(pred_node));
            char* obj;

            if (librdf_node_is_literal(obj_node))
            {
                obj = (char*) librdf_node_get_literal_value(obj_node);
                //triple->objType = ssElement_TYPE_LIT;
                resultTriple = new Triple(QString(sub), QString(pred), QString(obj), RdfTripleType::LITERAL);
            }
            else
            {
                obj = (char*) librdf_uri_as_string(librdf_node_get_uri(obj_node));
                //triple->objType = ssElement_TYPE_URI;
                resultTriple = new Triple(QString(sub), QString(pred), QString(obj), RdfTripleType::URI);
            }

            tripleList.append(resultTriple);

            librdf_stream_next(stream);
        }

        librdf_free_stream(stream);

    }

    Stopwatch::stopAndWrite("query");

    if (command->needConfirmation()) {
        CommandResponse* response = new CommandResponse(command, true);
        response->setTriples(tripleList);
        return response;
    }
    else {
        foreach (Triple* triple, tripleList) {
            delete triple;
        }
        return NULL;
    }
}

cutesib::CommandResponse *cutesib::BasicOperationHandler::remove(CommandRequest *command)
{
    QList<Triple*> removedTriples = rdfRemove(command->removeTriples());

    if (!removedTriples.isEmpty()) {
        CommandRequest *removeCommand = new CommandRequest(CommandType::REMOVE, this->id());
        removeCommand->setRemoveTriples(removedTriples);
        emit storeUpdated(removeCommand);
        delete removeCommand;
    }

    Stopwatch::stopAndWrite("remove");

    if (command->needConfirmation())
        return new CommandResponse(command, true);
    else
        return NULL;
}

cutesib::CommandResponse *cutesib::BasicOperationHandler::update(CommandRequest *command)
{
    QList<Triple*> removedTriples = rdfRemove(command->removeTriples());
    rdfInsert(command->insertTriples());

    CommandRequest *updateCommand = new CommandRequest(CommandType::UPDATE, this->id());
    updateCommand->setRemoveTriples(removedTriples);
    updateCommand->setInsertTriplesS(command->insertTriplesS());
    emit storeUpdated(updateCommand);
    delete updateCommand;

    Stopwatch::stopAndWrite("update");

    if (command->needConfirmation())
        return new CommandResponse(command, true);
    else
        return NULL;
}

cutesib::CommandResponse *cutesib::BasicOperationHandler::perform(cutesib::CommandRequest* command)
{
    qDebug() << "START PERFORM";
    CommandType type = command->Type();

    if (type == CommandType::INSERT) {

        qDebug() << "Insert triples: " << command->insertTriples().count();

        return insert(command);

    } else if (type == CommandType::REMOVE) {

        qDebug() << "Remove triples: " << command->removeTriples().count();

        return remove(command);

    } else if (type == CommandType::UPDATE) {

        qDebug() << "Update triples: " << command->insertTriples().count() << " " << command->removeTriples().count();

        return update(command);

    } else if (type == CommandType::QUERY) {

        qDebug() << "Query triples: " << command->queryTriples().count();

        CommandResponse* response = query(command);

        // PARALLEL OR SEQUENTIAL QUERY
        if (triplestore_->getConfig()->getTriplestoreParameter("parallel") == "query") {
            emit queryResult(response);
        }

        return response;
    }

    return NULL;
}

cutesib::Triple *cutesib::BasicOperationHandler::statementToTriple(librdf_statement *statement)
{
    librdf_node* sub_node  = librdf_statement_get_subject(statement);
    librdf_node* pred_node = librdf_statement_get_predicate(statement);
    librdf_node* obj_node  = librdf_statement_get_object(statement);
    Triple* triple = NULL;

    char* sub = (char*) librdf_uri_as_string( librdf_node_get_uri(sub_node));
    char* pred = (char*) librdf_uri_as_string( librdf_node_get_uri(pred_node));
    char* obj;

    if (librdf_node_is_literal(obj_node))
    {
        obj = (char*) librdf_node_get_literal_value(obj_node);
        //triple->objType = ssElement_TYPE_LIT;
        triple = new Triple(QString(sub), QString(pred), QString(obj), RdfTripleType::LITERAL);
    }
    else
    {
        obj = (char*) librdf_uri_as_string(librdf_node_get_uri(obj_node));
        //triple->objType = ssElement_TYPE_URI;
        triple = new Triple(QString(sub), QString(pred), QString(obj), RdfTripleType::URI);
    }

    return triple;
}

void cutesib::BasicOperationHandler::updateState(cutesib::CommandRequest *command)
{

}

void cutesib::BasicOperationHandler::rdfInsert(QList<cutesib::Triple *> insertTriples)
{
    //Rdf Redland Initialize..
    librdf_statement* statement ;
    ///////////////////////////

    foreach (Triple* triple, insertTriples) {
        //Insert statement
        statement=librdf_new_statement(RDF_world);

        librdf_statement_set_subject(statement,
                                     librdf_new_node_from_uri_string(RDF_world,  (const unsigned char*) triple->subject().toUtf8().data()));

        librdf_statement_set_predicate(statement,
                                       librdf_new_node_from_uri_string(RDF_world, (const unsigned char*) triple->predicate().toUtf8().data()));

        if (triple->objectType() == RdfTripleType::URI) {
            //URI: 	t->objType==0
            librdf_statement_set_object(statement,
                                        librdf_new_node_from_uri_string(RDF_world,  (const unsigned char*) triple->object().toUtf8().data()));
        }
        else
        {
            //Literal:  	t->objType==1
            librdf_statement_set_object(statement,
                                        librdf_new_node_from_literal(RDF_world, (const unsigned char*) triple->object().toUtf8().data(), NULL, 0));

        }

        /*if ( param->virtuoso)
        librdf_model_context_add_statement(param->RDF_model, param->virtuoso_context, statement);
    else*/
        librdf_model_add_statement(RDF_model, statement);


        librdf_free_statement(statement);
    }
}

QList<cutesib::Triple*> cutesib::BasicOperationHandler::rdfRemove(QList<cutesib::Triple *> removeTriples)
{
    librdf_statement* 	statement ;
    librdf_statement*	partial_statement;
    librdf_stream* 	stream = NULL;

    int al_least_one_wildcard = 0;
    int all_wildcard = 0;

    QList<Triple*> removedTriples;

    foreach (Triple* triple, removeTriples) {

        al_least_one_wildcard = 0;
        all_wildcard = 1;

        partial_statement=librdf_new_statement(RDF_world);

        if ( (strcmp((char*) triple->subject().toUtf8().data() ,wildcard1) ==0) || ( (strcmp((char*) triple->subject().toUtf8().data(),wildcard2) ==0) ))
        {	librdf_statement_set_subject(partial_statement, NULL);
            //fprintf (stderr,"wildcard subj enable ");
            al_least_one_wildcard=1;
        }
        else
        {	librdf_statement_set_subject(partial_statement, librdf_new_node_from_uri_string(RDF_world,  (const unsigned char*) triple->subject().toUtf8().data()));
            all_wildcard=0;
            //fprintf (stderr,"no wildcard subj enable");
        }

        if ( (strcmp((char*) triple->predicate().toUtf8().data(), wildcard1) ==0) || ( (strcmp((char*) triple->predicate().toUtf8().data(), wildcard2) ==0) ))
        {	librdf_statement_set_predicate(partial_statement, NULL);
            //fprintf (stderr,va"wildcard pred enable ");
            al_least_one_wildcard=1;
        }
        else
        {	librdf_statement_set_predicate(partial_statement, librdf_new_node_from_uri_string(RDF_world, (const unsigned char*) triple->predicate().toUtf8().data()));
        all_wildcard=0;
        }

        if ( (strcmp((char*) triple->object().toUtf8().data(), wildcard1) ==0) || ( (strcmp((char*) triple->object().toUtf8().data(), wildcard2) ==0) ))
        {	librdf_statement_set_object(partial_statement, NULL);
            //fprintf (stderr,"wildcard obj enable ");
            al_least_one_wildcard=1;
        }
        else
        {
            all_wildcard=0;
            if (triple->objectType() == RdfTripleType::URI)
            {
                librdf_statement_set_object(partial_statement,
                        librdf_new_node_from_uri_string(RDF_world, (const unsigned char*) triple->object().toUtf8().data()));
            }
            else
            {
                librdf_statement_set_object(partial_statement,
                        librdf_new_node_from_literal(RDF_world, (const unsigned char*) triple->object().toUtf8().data(), NULL, 0));
            }
        }

        //param->subs_sheduler_removed_triple=TRUE;

        if (al_least_one_wildcard)
        {
            if (all_wildcard)
            {
                //If the TS is big I can't copy all the content in ram.. so I need this solution.
                qDebug("ALL WC! RESET DB\n");
                //triplestore_->removeAllStatements();

                stream = librdf_model_as_stream(RDF_model);

            }
            else
            {
                //One or more WC
                /*if (param->virtuoso)
                    stream=librdf_model_find_statements_in_context(param->RDF_model, partial_statement,  param->virtuoso_context);
                else*/
                stream = librdf_model_find_statements(RDF_model, partial_statement);
            }

            if (stream != NULL) {
                while(!librdf_stream_end(stream))
                {
                    statement = librdf_stream_get_object(stream);
                    librdf_model_remove_statement(RDF_model, statement);

                    Triple* t;
                    t = statementToTriple(statement);
                    if (t != NULL)
                        removedTriples.append(t);

                    librdf_stream_next(stream);
                }
                librdf_free_stream(stream);
            }
        }
        else
        {
            //No WC


            qDebug("Removing\n");
            ///INFO: TO_IVAN comment for windows (fail in windows.)
            //librdf_statement_print(partial_statement , stdout);
            qDebug("\n");

            /*if (param->virtuoso)
                librdf_model_context_remove_statement(param->RDF_model, param->virtuoso_context, partial_statement);
            else*/
            librdf_model_remove_statement(RDF_model, partial_statement);
            removedTriples.append(triple->createCopy());
        }

        librdf_free_statement(partial_statement);
    }

    return removedTriples;
}
