/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   01.04.2015
 * @brief  Command class is base class to describe some operations that are performed by modules.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * Command is used to describe some operation or task for module.
 * Scheduler manages commands in the SIB. Modules dosen't send commands
 * directly beetwen each other. Commands are sent to scheduler and it
 * gets information from the command to resend the command to appropriate
 * reciever (other module).
 *
 * Contact: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2015-2016 by PetrSU.
 */


#ifndef CUTESIB_COMMAND_H
#define CUTESIB_COMMAND_H

#include <QtCore/QObject>
#include <QtCore/QVariant>
#include <QtCore/QMap>

#include <accesspointinterface.h>


namespace cutesib {

using namespace cutesib::access_point;

/**
 * @brief Command type.
 */
enum class  CommandType
{
    // Triple store commands for triples.
    INSERT = 1,     /**< Insert triples. */
    REMOVE,         /**< Remove triples. */
    UPDATE,         /**< Update triples. */
    QUERY,          /**< Query triples. */
    SPARQL_QUERY,   /**< SPARQL query. */

    // Subscription command (triples).
    SUBSCRIBE,              /**< Subscribe triples. For subscribe 'query' triples from command are used. */
    SUBSCRIPTION_INDICATE,  /**< Indicattion about changes for subscribed data. */
    UNSUBSCRIBE             /**< Unsubscribe subscription. */
};

/**
 * @brief Identifiers for command parameters.
 * Parameters can be set for commands.
 * @see Command
 */
enum class CommandParameter
{
    QUERY_TEXT,         ///< Value of parameter is a text (QString) that describes a some query.
    QUERY_RESULT_TEXT,  ///< Value of parameter is a text (QString) that describes a result of some query.
    SUBSCRIPTION_ID,     ///< Value of parameter is a text (QString) that describes a subscription id.

    // Used with SPARQL subscription.
    SUBSCRIPTION_NEW_RESULT,   ///< Value of parameter is a text (QString) that describes a new result (not riples) for subscription.
    SUBSCRIPTION_OLD_RESULT,    ///< Value of parameter is a text (QString) that describes a old result (not riples) for subscription.

    SSAP_MESSAGE,
    REQUEST
};

/**
 * @brief The Command class represents a command to communicate between SIB modules.
 * Modeles of the SIB create a command and send it to scheduler.
 * Scheduler based on command source ('form') and destination ('to') send a command to
 * appropriate module to perfom it. If the command must be confirmed then the destination
 * module signals scheduler about ending of performing and scheduler send it to source module.
 * This class cannot be created explicitly, it created with inherit classes.
 * @see CommandRequest
 * @see CommandResponse
 */
class Command : public QObject
{

    Q_OBJECT

public:
    /** @brief Command destructor */
    virtual ~Command();

    /**
     * @brief Gets destination module of the command.
     * Destination module processes a command
     * and sends back to source module a result if it needed.
     * @return module for processing command.
     */
    QString To() const { return destination_; }

    /**
     * @brief Gets source of the command.
     * @return module of the SIB which created a command.
     */
    QString From() const { return source_; }

    /**
     * @brief Gets a type of the command.
     * Command can contain internal SIB operation
     * or operation for triples store.
     * @return type of the command.
     */
    CommandType Type() const { return type_; }

    /**
     * @brief Sets a parameter for command.
     * If the parameter with given name is exists then it will be replaced.
     * @param parameter identifier.
     * @param value of the parameter.
     */
    void setParameter(CommandParameter parameter, QVariant value);

    /**
     * @brief Gets a parameter.
     * @param parameter identifier.
     * @return value of parameter on success or invalid QVariant (use QVariant::isValid()) otherwise (parameter is not exists).
     */
    QVariant getParameter(CommandParameter parameter);

    /**
     * @brief Removes a parameter.
     * @param parameter identifier.
     */
    void removesParameter(CommandParameter parameter);


protected:
    /**
     * @brief Command constructor.
     * @param type of the command.
     * @param from is a source of command.
     * @param to is a destination of command.
     * @param parent of the instance.
     */
    explicit Command(CommandType type, QString from, QString to, QObject *parent = 0);

    /**
     * @brief Command constructor without destination specified.
     * Command without destination processing by #Scheduler based commands type mapping from #SibConfig.
     * @param type of the command.
     * @param from is a source of command.
     * @param parent of the instance.
     */
    explicit Command(CommandType type, QString from, QObject *parent = 0);


private: // Fields
    QString source_;          ///< Source SIB module of the command.
    QString destination_;     ///< Destination SIB module.

    CommandType type_;        ///<  Type of the command.

    QMap<CommandParameter, QVariant> parameters_;     ///< Parameters of the command.
};

} // Namespace cutesib.

#endif // CUTESIB_COMMAND_H
