/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   25.09.2015
 * @brief  Request of command.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * CommandRequest is used when a module needs to delegate operation or send some notifications to other module.
 *
 * Contact: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2015-2016 by PetrSU.
 */


#ifndef CUTESIB_COMMANDREQUEST_H
#define CUTESIB_COMMANDREQUEST_H

#include <QtCore/QObject>
#include <QtCore/QDebug>
#include <QtCore/QSharedPointer>

#include "command.h"
#include "ssapmessage.h"


namespace cutesib {

/**
 * @brief The CommandRequest class is used to send command for performing between modules.
 * Request has three base parameters: triples for inserting, removing and querying. Triples for querying are also used for
 * subscription commands. Additional parameters can be added with corresponing method from #Command class.
 * @see Command
 */
class CommandRequest : public Command
{
    Q_OBJECT

public:
    /**
     * @brief CommandRequest constructor.
     * @param type of the command.
     * @param from is identificator of module who send command.
     * @param to is identificator of module which is responsive for command processing.
     * @param parent of this instance.
     * @see CommandType
     * @see SibModule
     */
    CommandRequest(CommandType type, QString from, QString to, bool needConfirmation = false, QObject *parent = 0);

    /**
     * @brief CommandRequest constructor without 'to'.
     * When destination (to) module is not specify then commad will be sent by #Scheduler based on
     * command type mappin from #SibConfig.
     * @param type of the command.
     * @param from is identificator of module who send command.
     * @param parent of this instance.
     * @see CommandType
     * @see SibModule
     */
    CommandRequest(CommandType type, QString from, bool needConfirmation = false, QObject *parent = 0);

    /**
     * @brief ~CommandRequest destructor.
     * Destructor deletes triples that are set for request.
     */
    virtual ~CommandRequest();

    /**
     * @brief Gets a confirmation flag.
     * If the confirmation is needed then destination module must
     * sends back to source module a result.
     * @return
     */
    bool needConfirmation() const;

    /**
     * @brief Sets a confirmation flag.
     * @param needConfirmation true/false flag.
     * @see needConfirmation()
     */
    void setConfirmation(bool needConfirmation);

    /**
     * @brief Sets triples as 'inserted' for request.
     * While performing command these triples will be inserted to triplestore.
     * @param triples that should be inserted.
     */
    void setInsertTriples(QList<Triple *> triples);

    /**
     * @brief Sets triples as 'removed' for request.
     * While performing command these triples will be removed from triplestore.
     * @param triples that should be removed.
     */
    void setRemoveTriples(QList<Triple *> triples);

    /**
     * @brief Sets triples as 'query' for request.
     * While performing command these triples will be used for query to get triples from triplestore.
     * These triples can be triples-patterns. For triple-pattern any value (* - mask) can be used for subject, predicate or object.
     * @param triples that are used to query triples.
     */
    void setQueryTriples(QList<Triple *> triples);

    /**
     * @brief Sets subscription identifier.
     * Ssbscription identifier is used with unsubscribe command.
     * @param id of the subscription.
     */
    void setSubscriptionId(QString id);

    /**
     * @brief Gets triples for inserting.
     * @return triples that should be inserted.
     */
    const QList<Triple *> insertTriples() const;

    /**
     * @brief Gets triples for removing.
     * @return triples that should be removed.
     */
    const QList<Triple *> removeTriples() const;

    /**
     * @brief Gets triples for querying.
     * @return triples that should be used queried.
     */
    const QList<Triple *> queryTriples() const;

    /**
     * @brief Gets triples for inserting in shared pointer.
     * @return triples that should be inserted.
     */
    const QSharedPointer<QList<Triple *>> insertTriplesS() const { return sharedInsertTriples_; }

    /**
     * @brief Gets triples for removing in shared pointer.
     * @return triples that should be removed.
     */
    const QSharedPointer<QList<Triple *>> removeTriplesS() const { return sharedRemoveTriples_; }

    /**
     * @brief Gets triples for querying in shared pointer.
     * @return triples that should be used for query.
     */
    const QSharedPointer<QList<Triple *>> queryTriplesS() const { return sharedQueryTriples_; }

    /**
     * @brief Sets triples for inserting in shared pointer.
     * @param triples that should be inserted.
     */
    void setInsertTriplesS(QSharedPointer<QList<Triple *>> triples);

    /**
     * @brief Sets triples for removing in shared pointer.
     * @param triples that should be removed.
     */
    void setRemoveTriplesS(QSharedPointer<QList<Triple *>> triples);

    /**
     * @brief Sets triples for querying in shared pointer.
     * @param triples that should be queried.
     */
    void setQueryTriplesS(QSharedPointer<QList<Triple *>> triples);

    /**
     * @brief Gets a subscription identifier.
     * @return subscription identifier.
     */
    const QString subscriptionId() const;

    SsapMessage* ssapMessage;
    QSharedPointer<cutesib::access_point::Request> request;



private: // Methods.
    // TODO: move or not to Triple class.
    /**
     * @brief Deliter for shared pointer with triples.
     * @param triples to be deleted.
     */
    static void deleteTriples(QList<Triple *>* triples);

private: // Fields.
    bool needConfirmation_;         ///< Flag: command must be confirmed or not.

    QList<Triple*> insertTriples_;  ///< Command parameter: triples for inserting.
    QList<Triple*> removeTriples_;  ///< Command parameter: triples for removing.
    QList<Triple*> queryTriples_;   ///< Command parameter: triples for querying.

    QSharedPointer<QList<Triple *>> sharedInsertTriples_;   ///< Command parameter: triples for inserting.
    QSharedPointer<QList<Triple *>> sharedRemoveTriples_;   ///< Command parameter: triples for removing.
    QSharedPointer<QList<Triple *>> sharedQueryTriples_;    ///< Command parameter: triples for querying.

    QString subscriptionId_;   ///< Command parameter: id of the subscription.
};

} // Namespace cutesib.

#endif // CUTESIB_COMMANDREQUEST_H
