/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   25.09.2015
 * @brief  Response for processed command.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * When a module has processed a command (CommandRequest) it should send a response (when needed).
 * Response is sent when for CommandRequest 'needConfirmation' field has a true value.
 *
 * Web-site: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2015-2016 by PetrSU.
 */


#ifndef CUTESIB_COMMANDRESPONSE_H
#define CUTESIB_COMMANDRESPONSE_H

#include <QtCore/QObject>

#include "command.h"
#include "commandrequest.h"
#include "ssapmessage.h"
#include "siberrors.h"


namespace cutesib {

/**
 * @brief The CommandResponse class is used for sending result of command processing.
 * Response has one base parameter: triples as command result.
 * Additional parameters can be added with corresponing method from #Command class.
 * @see Command
 */
class CommandResponse : public Command
{
    Q_OBJECT

public:
    /**
     * @brief CommandResponse constructor.
     * @param request for creating response.
     * @param isSuccesfull a flag: succesfully performed command or not.
     * @param parent of the instance.
     * @see getError
     */
    CommandResponse(CommandRequest *request, bool isSuccessful, QObject *parent = 0);

    /**
     * @brief CommandResponse constructor.
     * @param request for creating response.
     * @param error for performed command.
     * @param parent of the instance.
     * @see getError
     * @see CommandErrors
     */
    CommandResponse(CommandRequest *request, SibError error = SibError::NONE, QObject *parent = 0);

    /**
     * @brief ~CommandResponse destructor.
     * Destructor deletes triples that are set for response.
     */
    virtual ~CommandResponse();

    /**
     * @brief Gets request for current response.
     * @return request of initial command.
     */
    CommandRequest* getRequest() { return request_; }

    /**
     * @brief Gets flag: command performed successfully or not.
     * You can get a error code with #getError method.
     * @return true if the command successfully performed or false otherwise.
     */
    bool isSuccessful() { return isSuccessful_; }

    /**
     * @brief Gets an error code of performed command.
     * @return CommandErrors::NO for successfully performed command or error code otherwise.
     */
    inline SibError getError() { return errorCode_; }

    /**
     * @brief Sets triples list corresponding for command request (result of performed command).
     * @param triples to set.
     */
    void setTriples(QList<Triple *> triples);

    /**
     * @brief Gets triples list corresponding for command request (result of performed command).
     * You need to make a copy of triples to use it after response will be deleted.
     * @return list with triples.
     */
    const QList<Triple *> triples() const;


private: // Fields
    CommandRequest *request_;   ///< Initial request for which the response is  created.
    QList<Triple *> triples_;   ///< Triplse corresponding for command request (result of performed command).
    bool isSuccessful_;         ///< Flag: command performed succesfull or not.
    SibError errorCode_;   ///< Error code for unsuccessfully performed command.

};

} // Namespace cutesib.

#endif // CUTESIB_COMMANDRESPONSE_H
