/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   29.04.2016
 * @brief  Contains runtime context for the SIB.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * SIB context contains configuration of the SIB, and provides access
 * to core SIB modules (#Scheduler, #Triplestore) and to additional modules (#SibModule).
 *
 * Web-site: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2016 by PetrSU.
 */


#ifndef CUTESIB_CONTEXT_H
#define CUTESIB_CONTEXT_H

#include <QtCore/QObject>
#include <QtCore/QList>

#include "sibconfig.h"
#include "sibmodule.h"
#include "scheduler.h"
#include "triplestore.h"


namespace cutesib {

// Forward declaration.
class Scheduler;


/**
 * @brief The Context class is used to store information about SIB (config, modules, other elements).
 * Context is passed to modules and SIB elemts to initialize it.
 */
class Context : public QObject
{
    Q_OBJECT

public:
    /**
     * @brief Context class constructor.
     * @param parent of the instance.
     */
    Context(QObject *parent = 0);

    /**
     * @brief Context class destructor.
     */
    ~Context();

    /**
     * @brief Sets the scheduler.
     * @param scheduler.
     * @see Scheduler
     */
    void setScheduler(Scheduler *scheduler) { scheduler_ = scheduler; }

    /**
     * @brief Gets a scheduler of the SIB.
     * @return scheduler.
     * @see Scheduler
     */
    Scheduler* scheduler() { return scheduler_; }

    /**
     * @brief Sets the config.
     * @param config.
     * @see SibConfig
     */
    void setConfig(SibConfig* config) { config_ = config; }

    /**
     * @brief Gets the config of the SIB.
     * @return config.
     * @see SibConfig
     */
    SibConfig* config() { return config_; }

    /**
     * @brief Sets the triplestore.
     * @param triplestore.
     * @see Triplestore
     */
    void setTriplestore(Triplestore* triplestore) { triplestore_ = triplestore; }

    /**
     * @brief Gets the triplestore of the SIB.
     * @return triplestore.
     * @see Triplestore
     */
    Triplestore* triplestore() { return triplestore_; }

    /**
     * @brief Sets modules.
     * @param modules.
     * @see SibModule
     */
    void setModules(QList<SibModule*> modules) { modules_ = modules; }

    /**
     * @brief Gets modules of the SIB.
     * @return modules.
     * @see SibModule
     */
    QList<SibModule*> modules() { return modules_; }

    /**
     * @brief Get SIB module by identifier.
     * @param id identifier of module.
     * @return SIB module on success or nullptr otherwise (no modules with given id).
     */
    SibModule *getModule(QString id);

    /**
     * @brief Gets module by command type.
     * To get module mapping of modules and command types is used.
     * @param type of the command.
     * @return module that is mapped to type or nullptr when the mapping is not set.
     */
    cutesib::SibModule *getModule(CommandType type);

private: // Fields.
    Scheduler *scheduler_;      ///< Scheduler of the SIB.
    SibConfig *config_;         ///< Config of the SIB.
    Triplestore *triplestore_;  ///< Triplstore of the SIB.
    QList<SibModule*> modules_; ///< Modules of the SIB.
};

} // Namespace cutesib.

#endif // CUTESIB_CONTEXT_H
