/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   01.02.2016
 * @brief  Base class for operations handler.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * OperationHandler is base class for operations handlers. Handlers implement logic to perform
 * actions with triples and triplestore.
 *
 * There are
 * - 4 basic operation: 'insert', 'remove', 'update', 'query';
 * - 1 basic persistent operation: subscription.
 * All operation works with triples, but each operation (paramteres) can be implemented with different ways.
 * For example 'update' could be done with row triples, with SPARQL UPDATE or with RDF/XML.
 * For basic subscription operation triples-patters are used, but it can be extended with SPARQL SELECT.
 *
 * Each handler has an identificator. This identificator is mapped with some command type (operation).
 * Based on this mapping Scheduler manages commands with operations to send it to the appropriate handler.
 * In this way SIB supports extensibility to perform new operations by including new handlers.
 *
 * Contact: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright 2016 (C) by PetrSU.
 */


#ifndef CUTESIB_OPERATIONHANDLER_H
#define CUTESIB_OPERATIONHANDLER_H

#include <QObject>

#include "sibmodule.h"
#include "commandrequest.h"
#include "commandresponse.h"


namespace cutesib {

/**
 * @brief The OperationHandler class is base class for operations handlers.
 * Handlers are implemented logic to perform actions with triples and triplestore.
 *
 * Each handler has an identificator. This identificator is mapped with some operation.
 * Based on this mapping Scheduler manages commands with operations to send it
 * to the appropriate handler.
 * Handlers has one method to perform commands with operations, one signal
 * to inform other handlers about store updating and one slot to get notification
 * about triplestore updating. It is important for subscription operations.
 *
 * Handlers doesn't know about other handlers. All configuation and signal/slot registration
 * are done by SIB (see #SibManager) on initializtion stage (see #isUpdatesNotificationNeeded method).
 */
class OperationHandler : public SibModule
{

    Q_OBJECT

public:
    /**
     * @brief OperationHandler constructor.
     * @param config of the SIB to get needed parameters.
     * @param parent of the instance.
     */
    explicit OperationHandler(QString id, bool needUpdateNotifications = false, QObject *parent = 0);

    /**
     * @brief ~OperationHandler destructor.
     */
    virtual ~OperationHandler();

    /**
     * @brief Gets a flag: handler needs to know about triplestore updating by other handlers.
     * @return true if updating is important or false otherwise.
     */
    bool needUpdateNotifications();

    /**
     * @brief Performs a command with operation.
     * @param command with operation to perform.
     * @return response for command with a result of performing operation
     * if the command does not requared response then returns NULL.
     * @see Command
     * @see CommandRequest
     * @see CommandResponse
     */
    virtual  CommandResponse* perform(CommandRequest *command) = 0;


signals:
    /**
     * @brief Signal to notify about triplestore updating.
     * When a handler updates triplestore it send this signal. Command as a parameter has
     * information about changes (for example triples that were inserted, removed from triples store).
     * @param command with operation under triplestore which led to updates.
     * @see CommandRequest
     */
    void storeUpdated(CommandRequest *command);


public slots:
    /**
     * @brief Slot to get signal about triplestore updates.
     * This slot registered with signals by SIB (#SibManager) with signals
     * when the #isUpdatedImportant() returns true.
     * @param command with operation under triplestore which led to updates.
     * @see isUpdatedImportant()
     * @see CommandRequest
     * @see SibManager
     */
    virtual void updateState(CommandRequest *command) = 0;


private: // Fields.
    bool needUpdateNotifications_;   ///< Flag: handler needs to know about triplestore updating by other handlers.
};

} // Namespace cutesib.


#endif // CUTESIB_OPERATIONHANDLER_H
