/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   10.03.2016
 * @brief  Base class for protocol handlers.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * ProtocolHandler is base class for protocol handlers. Handlers implemente logic for application protocols.
 * Handlers work in pair with access points. Each handler could have several access points as a sources
 * of clients (agents) requests and for sending responses.
 * Each imlemented ProtocolHandler have an indetifier, this identifier is used in config file and in the SIB
 * to configure handler and control it. Identifier must be uniq for every ProtocolHandler in the SIB.
 *
 * Web-site: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2015-2016 by PetrSU.
 */


#ifndef CUTESIB_PROTOCOLHANDLER_H
#define CUTESIB_PROTOCOLHANDLER_H

#include <QObject>
#include <QList>

#include "sibconfig.h"
#include "sibmodule.h"
#include "accesspointinterface.h"
#include "command.h"
#include "commandrequest.h"
#include "commandresponse.h"


namespace cutesib
{

/**
 * @brief The ProtocolHandler class is base class for handlers which implement protocols.
 * Access to the SIB is implemented by access points.
 * Each access point processes connection/disconnection of agent, receiving of requests and sending responses.
 * Handler works with access points and provides requests processing and response creating (protocol logic).
 * Handler also works with #Scheduler passing commands to #OperationHandler.
 *
 * Handlers are initializeb by #SibManager. SibManager creates handlers and sets to them assess points.
 * After initialization SibManager starts the handler.
 *
 * @see Scheduler
 * @see SibManager
 * @see AccessPointInterface
 */
class ProtocolHandler : public SibModule
{
    Q_OBJECT
public:
    /**
     * @brief ProtocolHandler constructor.
     * Creates handler with given id and supported protocols;
     * Protocols are used to connect handler with appropriate access points.
     * Id/names of protocols can be found in files with Access Point Interface.
     * @param id is an identifier of the handler, identifier is used in config file and in the SIB
     * to configure handler and control it.
     * @param protocols supported by handler.
     * @param parent of the instance.
     * @see protocols
     */
    explicit ProtocolHandler(QString id, QStringList protocols, QObject *parent = 0);

    /**
     * @brief ProtocolHandler destructor.
     */
    virtual ~ProtocolHandler();

    /**
     * @brief Connect access point to handler.
     * Access point process connection with agent: receives requests and sends response.
     * Hadler should receives request from access point, process requests and sends
     * responses through access point.
     * Several access points can be connected to handler.
     *
     * SIB guarantees calling this method only for stopped handlers (before #start or after #stop method).
     * @param accessPoint to connect.
     * @param isAsync flag to use access point in main (SIB) thread or other thread.
     * @return true on succesfull connection or false otherwise.
     * @see AccessPointInterface
     */
    virtual bool connectAccessPoint(access_point::AccessPointInterface *accessPoint, bool isAsync = false) = 0;

    /**
     * @brief Disconnect all access points from handler.
     * SIB guarantees calling this method only for stopped handlers (before #start or after #stop method).
     */
    virtual void disconnectAccessPoints() = 0;

    /**
     * @brief Starts the handler.
     * Handler starts to process request from access points.
     * Before callin this method handler ignores all requests from acces points.
     * @return true on success or false otherwise.
     */
    virtual bool start() { return true; }

    /**
     * @brief Stop the handler.
     * Handler stops to process request from access points.
     */
    virtual void stop() { }

    /**
     * @brief Shutdowns the handler.
     * Stops all activity of handler, disconnects all access points and
     * prepares handler for clean deletion.
     * After calling this method handler will not be used any more and destructor will be
     * 'called'. SIB can call this method in any time, but only once.
     */
    virtual void shutdown() { }

    /**
     * @brief Gets protocols supported by handler.
     * Protocols are used to connect handler with appropriate access points.
     * Id/names of protocols can be found in files with Access Point Interface.
     * @return list with protocols id/names.
     */
     QStringList protocols() { return protocols_; }


private:
    QStringList protocols_; ///< Supported protools by handler.
};

} // Namespace cutesib.

#endif // CUTESIB_PROTOCOLHANDLER_H
