/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   29.04.2016
 * @brief  Base class for SIB modules.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * SIB uses modules to implement functionality.
 * Each module have identifier and special method to perform commands (#CommandRequest).
 * Idetifier is associated with one or several command types.
 * #Scheduler based on association between module (id) and command (command type) forward
 * command to module for performing the command.
 * #CommandRequest contains information to perform operation (name. parameters etc.) and
 * module makes action to processing command and sends response (#CommandResponse).
 *
 * Web-site: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2016 by PetrSU.
 */


#ifndef CUTESIB_SIBMODULE_H
#define CUTESIB_SIBMODULE_H

#include <QObject>
#include <QtCore/QString>

#include "commandrequest.h"
#include "commandresponse.h"
#include "siberrors.h"


namespace cutesib {

// Forward declaration.
class Context;

/**
 * @brief The SibModule class is a base class for all modules to implement SIB functionality.
 * SIB uses modules to implement functionality.
 * Each module have identifier and special method to perform commands (#CommandRequest).
 * Idetifier is associated with one or several command types.
 * #Scheduler based on association between module (id) and command (command type) forward
 * command to module for performing the command.
 * #CommandRequest contains information to perform operation (name. parameters etc.) and
 * module makes action to processing command and sends response (#CommandResponse).
 *
 * @see CommandRequest
 * @see CommandResponse
 * @see Scheduler
 */
class SibModule : public QObject
{
    Q_OBJECT

public:
    /**
     * @brief SibModule constructor.
     * @param id of the module. Should be uniq in the SIB.
     * @param parent of the instance.
     */
    explicit SibModule(QString id,  QObject *parent = 0);

    /**
     * @brief SibModule destructor.
     */
    virtual ~SibModule();

    /**
     * @brief Initializes module.
     * All configuration should be done
     * in this method to prepare module for working (#start).
     * When this method is override by child class then call superclass method
     * to set ontext field (see #context).
     * @param context of SIB.
     * @return SibError::NONE on success or not otherwise.
     * @see Context
     */
    virtual SibError initialize(Context *context);

    /**
     * @brief Starts the module.
     * Module starts to work.
     * @return true on success or false otherwise.
     */
    virtual bool start() { return true; }

    /**
     * @brief Stops the module.
     * Module stops to work.
     */
    virtual void stop() { }

    /**
     * @brief Shutdowns the module.
     * Stops all activity of the module and prepares module to delete.
     * After calling this method module will not be used any more and destructor will be
     * 'called'. SIB can call this method in any time, but only once.
     */
    virtual void shutdown() { }

    /**
     * @brief Gets a context of the SIB.
     * @return context.
     * @see Context
     */
    inline Context* context() { return context_; }

    /**
     * @brief Getter for the module identifier.
     * Should be uniq in the SIB.
     * @return identifier of the module.
     */
    inline QString id() { return id_; }

    /**
     * @brief Performs a command.
     * This method is called twhen command (#CommandRequest) should be performed by module.
     * Module can perform several commands. Each #CommandRequest have a type, this
     * type is used by module to determine what actions should be done to perform the command.
     * #CommandRequest has a confiramtion flag. If this flag equals true then module should
     * create a response (#CommandResponse) and returns it. If the flag equals false then
     * module should delete a equest and returns nullptr;
     *
     * @param command to perform.
     * @return response to the command when confirmation is needed and nullptr when not needed.
     */
    virtual CommandResponse* perform(CommandRequest *command) = 0;


private: // Fields.
    QString id_;        ///< Identifier of the module.
    Context *context_;  ///< SIB context.
};

} // Namespace cutesib.


#endif // CUTESIB_SIBMODULE_H


