/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   06.05.2015
 * @brief  Represents a RDF-triple.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * RDF triple consists of three elements: subject, predicate and object.
 * Predicate is also named property.
 * You can read more about RDF triples here: https://www.w3.org/TR/rdf11-primer/
 *
 * Now in the SIB the blank nodes (of RDF-graph) are not used that's why subject
 * is always URI and object can be URI or literal.
 *
 * Web-site: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2015-2016 by PetrSU.
 */


#include "triple.h"

#include <QtCore/QDebug>


/*****************************************************************************/
/*************************** Definitions/Constants ***************************/

//  Definition of the constants variable for Any-URI.
const QString cutesib::Triple::AnyUri = "http://www.nokia.com/NRC/M3/sib#any";
const QString cutesib::Triple::AnyShortForm = "sib:any";

int cutesib::Triple::instanceCounter = 0;


/*****************************************************************************/
/************************** Constructors/destructors *************************/

cutesib::Triple::Triple(QString subject, QString predicate, QString object, RdfTripleType objecType)
{
    subject_ = (subject == AnyShortForm) ? AnyUri: subject;
    predicate_ = (predicate == AnyShortForm) ? AnyUri : predicate;

    object_ = object;

    if (objecType == RdfTripleType::URI && object == AnyShortForm) {
        object_ = AnyUri;
    }

    objectType_ = objecType;

    ++instanceCounter;
    qDebug() << "Triples: " << instanceCounter;
}


cutesib::Triple::~Triple()
{
    --instanceCounter;
    qDebug() << "Triples: " << instanceCounter;
}


/*****************************************************************************/
/****************************** Public methods *******************************/

cutesib::Triple *cutesib::Triple::createCopy()
{
    return new Triple(subject_, predicate_, object_, objectType_);
}
