/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   02.03.2015
 * @brief  Access point that implements TCP protocol.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * Access point that implements TCP protocol.
 *
 * Contact: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2015-2016 by PetrSU.
 */


#include "tcpaccesspoint.h"

#include <iostream>

#include <QtCore/QObject>
#include <QtCore/QStringList>
#include <QtCore/QtPlugin>
#include <QtCore/QDebug>

#include "tcpclient.h"


/*****************************************************************************/
/*************************** Definitions/Constants ***************************/

const QByteArray cutesib::access_point::TcpAccessPoint::EmptyString(" ");
const QString cutesib::access_point::TcpAccessPoint::DataProtocol(CUTESIB_PROTOCOL_SSAPXML);

const QChar cutesib::access_point::TcpAccessPoint::ParameterDelimeter(',');
const QString cutesib::access_point::TcpAccessPoint::ParameterPort("port:");


/*****************************************************************************/
/************************** Constructors/destructors *************************/

cutesib::access_point::TcpAccessPoint::TcpAccessPoint(QObject *parent, int port) : QObject(parent)
{
    port_ = port;

    // Whenever a user connects, it will emit signal.
    connect(&server_, SIGNAL(newConnection()), this, SLOT(newConnection()));

    // Setup timer for checking client connections.
    checkingClientTimer_.setInterval(CheckingClientTimeout);
    connect(&checkingClientTimer_, SIGNAL(timeout()), this, SLOT(connectionChecking()));
}


/*****************************************************************************/
/****************************** Public methods *******************************/

void cutesib::access_point::TcpAccessPoint::initialize(QString id, QString parameters)
{
    id_ = id;

    QStringList parametersList = parameters.split(ParameterDelimeter);

    foreach (QString parameter, parametersList) {
        if (parameter.startsWith(ParameterPort)){
            setPortFromParameter(parameter);
        }
    }
}


QString cutesib::access_point::TcpAccessPoint::id()
{
    return id_;
}


int cutesib::access_point::TcpAccessPoint::start()
{
    // If the TCP server is already in listening mode then do nothing.
    // If no then start listening on specific port.
    if (server_.isListening()) {
        return -1;
    }

    if(!server_.listen(QHostAddress::Any, port_)) {
        qDebug() << "TCP access point could not start " << server_.errorString() << id();
        return -1;
    }

    qDebug() << "TCP access point " << id() << " started on port" << port_;

    return 0;
}


void cutesib::access_point::TcpAccessPoint::stop()
{
    if (!server_.isListening()) {
        return;
    }

    qDebug() << id();

    // Close all socets and then the TCP server.
    foreach (QSharedPointer<TcpClient> client, clients_) {
        QObject::disconnect(client->getSocket(), SIGNAL(disconnected()), this, SLOT(socketDisconnected()));
        QObject::disconnect(client->getSocket(), SIGNAL(readyRead()), this, SLOT(readyRead()));
    }

    clients_.clear();
    server_.close();
}


void cutesib::access_point::TcpAccessPoint::shutdown()
{
    stop();
}


void cutesib::access_point::TcpAccessPoint::send(const Client *client, const QByteArray data)
{
    TcpClient *tcpClient = dynamic_cast<TcpClient*>((Client*)client);

    if (!tcpClient) {
        qWarning() << "Incorrect client.";
        return;
    }

    tcpClient->Lock();
    send(tcpClient, data, -1); // Waits without timeout
    tcpClient->Unlock();
}


bool cutesib::access_point::TcpAccessPoint::isConnected(cutesib::access_point::Client *client)
{
    TcpClient *tcpClient = dynamic_cast<TcpClient*>(client);

    if (!tcpClient) {
        qWarning() << "Incorrect client.";
        return false;
    }

    tcpClient->getSocket()->write(EmptyString.data(), 1);
    tcpClient->getSocket()->flush();
    tcpClient->getSocket()->waitForBytesWritten(5000);

    return tcpClient->isConnected();
}


void cutesib::access_point::TcpAccessPoint::startChecking(QSharedPointer<Client> client)
{
    auto tcpClient = client.dynamicCast<TcpClient>();

    if (!tcpClient) {
        qWarning() << "Incorrect client.";
        return;
    }

    // Block checking clients to add a new one
    // and start timer to interval checking.
    // Timer works only if there are client to check connections.
    checkingClientLock_.lock();

    if (!checkingClients_.contains(tcpClient)) {
        checkingClients_.insert(tcpClient);
        qDebug() << "Checking clients:" << checkingClients_.count();
    }

    if (!checkingClientTimer_.isActive()) {
        checkingClientTimer_.start();
    }

    checkingClientLock_.unlock();
}


void cutesib::access_point::TcpAccessPoint::stopChecking(QSharedPointer<Client> client)
{
    auto tcpClient = client.dynamicCast<TcpClient>();

    if (!tcpClient) {
        qWarning() << "Incorrect client.";
        return;
    }

    // Block checking clients and remove client from checking list.
    // If there are no clients, then the timer will be stopped.
    checkingClientLock_.lock();

    checkingClients_.remove(tcpClient);

    if (checkingClients_.count() == 0)
        checkingClientTimer_.stop();

    checkingClientLock_.unlock();
}


void cutesib::access_point::TcpAccessPoint::close(Client* client)
{
    TcpClient* tcpClient = dynamic_cast<TcpClient*>(client);

    if (!tcpClient) {
        qWarning() << "Incorrect client.";
        return;
    }

    QObject::disconnect(tcpClient->getSocket(), SIGNAL(disconnected()), this, SLOT(socketDisconnected()));
    QObject::disconnect(tcpClient->getSocket(), SIGNAL(readyRead()), this, SLOT(readyRead()));

    qDebug() << "Connection closed with: " << tcpClient->getSocket()->peerAddress() <<  tcpClient->getSocket()->peerPort();

    tcpClient->getSocket()->close();

    auto removeClient = getTcpClient(tcpClient->getSocket());
    clients_.remove(removeClient);
}


/*****************************************************************************/
/******************************* Public slots ********************************/

void cutesib::access_point::TcpAccessPoint::newConnection()
{
    // Grab a new socket, create a client based on it
    // and connect apropriate slots to socket's signals.
    QTcpSocket *socket = server_.nextPendingConnection();

    qDebug() << "Client connected:"
             << socket->peerAddress() << socket->peerPort();

    QSharedPointer<TcpClient> client(new TcpClient(this, socket), &QObject::deleteLater);

    clients_.insert(client);

    connect(socket, SIGNAL(disconnected()), this, SLOT(socketDisconnected()));
    connect(socket, SIGNAL(readyRead()), this, SLOT(readyRead()));

    emit clientConnected(client);
}


void cutesib::access_point::TcpAccessPoint::readyRead()
{
    QTcpSocket* socket = qobject_cast<QTcpSocket*>(QObject::sender());

    auto sendClient = getTcpClient(socket);

    sendClient->addMessageData(socket->readAll());

    qDebug() << "Receved message from:" << socket->peerAddress() << socket->peerPort();

    // Send messages to SIB while there are full messages.
    while (sendClient->isReceivedFullMessage()) {
        auto request = QSharedPointer<Request>(new Request(this, sendClient, DataProtocol, sendClient->getMessage()));
        qDebug() << "Message to SIB:" << request->data();
        emit requestReceived(request);
    }
}


void cutesib::access_point::TcpAccessPoint::socketDisconnected()
{
    QTcpSocket* socket = dynamic_cast<QTcpSocket*> (QObject::sender());

    qDebug() << "Cleint disconnected:"
             << socket->peerAddress()
             << socket->peerPort();

    auto client = getTcpClient(socket);
    client->isConnected(false);

    checkingClientLock_.lock();
    clients_.remove(client);
    checkingClients_.remove(client);
    checkingClientLock_.unlock();

    emit clientDisconnected(client);
}


/*****************************************************************************/
/***************************** Private methods *******************************/

bool cutesib::access_point::TcpAccessPoint::send(TcpClient *client, const QByteArray data, int timeout)
{
    Q_UNUSED(timeout);
    qDebug() << "Send bytes to"
             << client->getSocket()->peerAddress()
             << client->getSocket()->peerPort()
             << "\nData:" << data << "\n";

    client->setMessage(data);
    QByteArray dataPart = client->getMessagePart(1000);


    while (!dataPart.isEmpty()) {
            qDebug() << "Send:" << dataPart;
        client->getSocket()->write(dataPart);
        client->getSocket()->flush();
        client->getSocket()->waitForBytesWritten(10000);

        dataPart = client->getMessagePart(1000);
    }

    return client->isConnected();
}


QSharedPointer<cutesib::access_point::TcpClient> cutesib::access_point::TcpAccessPoint::getTcpClient(const QTcpSocket *socket)
{
    foreach (QSharedPointer<TcpClient> client, clients_) {
        if (client->getSocket() == socket) {
            return client;
        }
    }

    return QSharedPointer<TcpClient>();
}


void cutesib::access_point::TcpAccessPoint::setPortFromParameter(QString parameter)
{
    int port = parameter.remove(0, 5).toInt();

    if (port > 0) {
        port_ = port;
    }
}


void cutesib::access_point::TcpAccessPoint::connectionChecking()
{
    int i = 0;

    qDebug() << "Connection checking for" << checkingClients_.count() << "clients.";

    // Check connections.
    // If the connections with client is lost,
    // then the signal is emitted and client will be removed from checking.
    // Cycle works while there are client for checking.
    while(true) {
        checkingClientLock_.lock();

        if (checkingClients_.empty()) {
            checkingClientTimer_.stop();
            checkingClientLock_.unlock();
            break;
        } else if (i >= checkingClients_.count()){
            checkingClientLock_.unlock();
            break;
        }

        auto client = checkingClients_.values().at(i);

        qDebug() << "Checking for"
                 << client->getSocket()->peerAddress()
                 << client->getSocket()->peerPort();

        client->Lock();
        bool connected = isConnected(client.data());
        client->Unlock();
        if (!connected) {
            qDebug() << "Connection is lost with"
                     << client->getSocket()->peerAddress()
                     << client->getSocket()->peerPort();
            checkingClients_.remove(client);
            clients_.remove(client);
            emit clientDisconnected(client);
        }

        checkingClientLock_.unlock();

        ++i;
    }
}

