/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   02.03.2015
 * @brief  Access point that implements TCP protocol.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * Access point that implements TCP protocol.
 *
 * Contact: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2015-2016 by PetrSU.
 */


#ifndef CUTESIB_TCPACCESSPOINT_H
#define CUTESIB_TCPACCESSPOINT_H


#include "tcpaccesspoint_global.h"

#include <QtCore/QSet>
#include <QtCore/QTimer>
#include <QtCore/QThread>
#include <QtCore/QMutex>

#include <QtNetwork/QTcpSocket>
#include <QtNetwork/QTcpServer>

#include <QtCore/QDebug>

#include "accesspointinterface.h"
#include "tcpclient.h"

// Plugin identifier of the access point.
#define TcpAccessPoint_iid "org.fruct.oss.Projects.CuteSIB.TcpAccessPoint"
#define CUTESIB_TCPAP_SSAPXML_PLUGIN_FILE "tcp_access_point.json"

// Plugin info.
#define CUTESIB_TCPAP_SSAPXML_ID "AP-TCP-SSAPXML"
#define CUTESIB_TCPAP_SSAPXML_DESCRITION "TCP Access Point for CuteSIB"
#define CUTESIB_TCPAP_SSAPXML_PROTOCOLS CUTESIB_PROTOCOL_SSAPXML
#define CUTESIB_TCPAP_SSAPXML_VERSION CUTESIB_TCPAP_VERSION


namespace cutesib {
namespace access_point {

// Forward declaration.
class TcpClient;

/**
 * @brief The TcpAccessPoint class implements access point interface
 * and supports TCP protocol to access SIB.
 * @see AccessPointInterface
 */
class TCPACCESSPOINTSHARED_EXPORT TcpAccessPoint: public QObject, public AccessPointInterface
{
    Q_OBJECT
    Q_INTERFACES(cutesib::access_point::AccessPointInterface)

public:
    /**
     * @brief TcpAccessPoint class constructor.
     * @param parent for the instance.
     * @param port is a TCP port to listen, default is 10010.
     */
    explicit TcpAccessPoint(QObject *parent = 0, int port = 10010);

    /**
     * @brief TcpAccessPoint class destructor.
     */
    ~TcpAccessPoint() { qDebug() << "TcpAccessPoint descructor"; }

    /**
     * @brief Sets parameters of the access point.
     * Format of parameters is key-value pairs: 'key:value,key:value,...'.
     * Now there is one parameter: port (default value is 10010).
     * @param id for access point.
     * @param parameters from SIB to access point.
     */
    void initialize(QString id, QString parameters);

    /**
     * @brief Gets access point identifier.
     * @return identifier.
     * @see setParameters
     */
    QString id();

    /**
     * @brief Starts the access point.
     * Starts to listen TCP port for client connection.
     * @return 0 on success or -1 otherwise.
     */
    int start();

    /**
     * @brief Stops work of the access point.
     * Stops to listen TCP port and to check network connections for clients.
     */
    void stop();

    /**
     * @brief Shutdowns the access point.
     */
    void shutdown();

    /**
     * @brief Sends data to client.
     * @param[in] client. Client to send the data.
     * @param[in] data. Data to send.
     * @see Client
     */
    void send(const Client* client, const QByteArray data);

    /**
     * @brief Checks client state: connected or not.
     * @param[in] client. Client to check connection.
     * @return true when the client is connected or false otherwise.
     */
    bool isConnected(Client *client);

    /**
     * @brief Starts to check connection state of a client (KeepAlive)
     * The client will be checking with some frequency.
     * When the client is no longer available, then the signal
     * (#clientDisconnected()) will be emitted.
     * The method does nothing if client connection state is already checking.
     * @param[in] client. Client to start checking connection.
     * @see clientDisconnected()
     */
    void startChecking(QSharedPointer<Client> client);

    /**
     * @brief Stops to check connection state of a client.
     * The method does nothing if client connection state is not checking.
     * @param[in] client. Client to stop checking connection.
     */
    void stopChecking(QSharedPointer<Client> client);

    /**
     * @brief Closes a connection with a client.
     * With closing connection a client will be removed from access point and deleted.
     * The client cannot be used for other action anymore.
     * @param[in] client. Client to close the connection.
     * @see Client
     */
    void close(Client* client);


public slots:
    /**
     * @brief Slot to process a new TCP connections.
     */
    void newConnection();

    /**
     * @brief Slot to get data when a client sends a request.
     */
    void readyRead();

    /**
     * @brief Slot to process client disconnection from the access point.
     */
    void socketDisconnected();


signals:
    /**
     * @brief Signal is emitted when a client disconnected from access point.
     * @param client that was disconnected.
     */
    void clientDisconnected(QSharedPointer<cutesib::access_point::Client> client);

    /**
     * @brief Signal is emitted when a client connected to access point.
     * @param client that was connected.
     */
    void clientConnected(QSharedPointer<cutesib::access_point::Client> client);

    /**
     * @brief Signal is emitted when a request from client is recieved.
     * @param request from client.
     */
    void requestReceived(QSharedPointer<cutesib::access_point::Request> request);

    /**
     * @brief Signals when the access poin stops working.
     * SIB uses this signal when access poin working in own thread (async AP).
     */
    void stopped();


private: // Methods
    /**
     * @brief Sends data to client with given timeout.
     * @param client to send the data.
     * @param data to send.
     * @param timeout to wait in milliseconds.
     * @see TcpClient
     */
    bool send(TcpClient *client, const QByteArray data, int timeout);

    /**
     * @brief Gets a client by socket.
     * @param socket of client.
     * @return client with given socket or 0 if the client is new.
     * @see TcpClient
     */
    QSharedPointer<TcpClient> getTcpClient(const QTcpSocket *socket);

    /**
     * @brief Sets the listen port for access point from parameter.
     * @param parameter from the SIB.
     */
    void setPortFromParameter(QString parameter);


private slots:
    /**
     * @brief Slot checks connections state for clients.
     * This slot is used with a timer.
     */
    void connectionChecking();


private: // Fields
    const static QByteArray EmptyString;   ///< Empty string to check client network state (KeepAlive).
    const static QString DataProtocol;     ///< Protocol name for data in the request (SSAP/XML).

    const static QString ParameterPort;     ///< Name of port parameter.
    const static QChar ParameterDelimeter;  ///< Delimeter for parameters.

    QTcpServer server_;         ///< Server to process TCP connections.
    QSet<QSharedPointer<TcpClient>> clients_;  ///< TCP clients.
    int port_;                  ///< Port to listen TCP connections.

    QTimer checkingClientTimer_;            ///< Timer to check clients connection state.
    QMutex checkingClientLock_;             ///< Lock for checking clients.
    QSet<QSharedPointer<TcpClient>> checkingClients_;      ///< TCP clients to check connections.
    static const int CheckingClientTimeout = 5000; ///< Interval timeout for timer.

    QString id_;   ///< Id that is gived by SIB to control access point.
};

} // namespace access_point
} // namespace cutesib

#endif // CUTESIB_TCPACCESSPOINT_H
