/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   05.03.2015
 * @brief  Class for TCP client.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * Class for TCP client.
 *
 * Contact: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2015 by PetrSU.
 */

#include "tcpclient.h"

#include <QtCore/QBuffer>
#include <QtCore/QDebug>

#include "accesspointinterface.h"


/*****************************************************************************/
/*************************** Definitions/Constants ***************************/

const QByteArray cutesib::access_point::TcpClient::MessageEnding = "</SSAP_message>";


/*****************************************************************************/
/************************** Constructors/destructors *************************/

cutesib::access_point::TcpClient::TcpClient(TcpAccessPoint *accessPoint, QTcpSocket *socket)
{
    socket_ = socket;
    accessPoint_ = accessPoint;
//    setParent((QObject *)accessPoint);
    isConnected_ = true;
}

cutesib::access_point::TcpClient::~TcpClient()
{
    if (socket_->isOpen()){
        socket_->close();
    }

    socket_->deleteLater();
}


/*****************************************************************************/
/****************************** Public methods *******************************/

QTcpSocket *cutesib::access_point::TcpClient::getSocket() const
{
    return socket_;
}

cutesib::access_point::AccessPointInterface *cutesib::access_point::TcpClient::getAccessPoint()
{
    return accessPoint_;
}


bool cutesib::access_point::TcpClient::isReceivedFullMessage()
{
    return receiveBuffer_.contains(MessageEnding);
}


void cutesib::access_point::TcpClient::addMessageData(QByteArray data)
{
    qDebug() << "Buffer:" << receiveBuffer_;
    receiveBuffer_.append(data);
    qDebug() << "New buffer:" << receiveBuffer_;
}


QByteArray cutesib::access_point::TcpClient::getMessage()
{
    int index = receiveBuffer_.indexOf(MessageEnding);

    if (index < 0) {
        return QByteArray();
    }

    index += MessageEnding.length();

    QByteArray temp;
    temp.append(receiveBuffer_.data(), index);

    receiveBuffer_ = receiveBuffer_.remove(0, index);

    return temp;
}

int cutesib::access_point::TcpClient::send(QByteArray message)
{
    qDebug() << "before send\n";
    socketLock_.lock();
    accessPoint_->send(this, message);
    socketLock_.unlock();

    return 0;
}

void cutesib::access_point::TcpClient::close()
{
    accessPoint_->close(this);
}


void cutesib::access_point::TcpClient::Lock()
{
    //socketLock_.lock();
}


void cutesib::access_point::TcpClient::Unlock()
{
    //socketLock_.unlock();
}

void cutesib::access_point::TcpClient::setMessage(QByteArray data)
{
    if (messageBuffer_.isOpen()) {
        messageBuffer_.close();
    }

    messageBuffer_.setData(data);

    messageBuffer_.open(QIODevice::ReadOnly);
}


QByteArray cutesib::access_point::TcpClient::getMessagePart(qint64 size)
{
    if (!messageBuffer_.isOpen()) {
        messageBuffer_.open(QIODevice::ReadOnly);
    }

    return messageBuffer_.read(size);
}

